/*
	Sim2D... the file containing "main" and the simulation doLoop for a 2D psuedo-biological simulation
*/

import java.text.*;
import java.io.*;

public class Sim2D {
	
	static WorldFrame theFrame;
	// the pixel world dimensions
	static int width = 700;
	static int height = 700;
	
	// the virtual world dimensions
	static final double xDimension = 100;		// x dimension in unspecified units
	static final double yDimension = 100;		// y dimension
	
	static double deltaT = 0.001;			// time-step
	static double simulationTime = 0;		// keeps track of current time
	static double runTime = 100.0;			// final time to run to
	static int counter = 0;					// keeps track of current integration step number
	
	// for file writing
	static boolean remote = false;			// no graphics 'cause running on server
	static int remoteCounter = 0;				// for counting integration steps between remote reports
	static int remoteReportStep = 100;		// number of integration steps between printed info reports
	static boolean infoWrite = false;			// write info to file flag
	static int infoWriteCounter = 0;
	static int infoWriteStep = 100;			// number of integration steps between info reports to file
	static boolean makeMovie = false;
	static int movieStep = 100;				// number of integration steps between movies
	static int movieCounter = 0;				// for counting integration steps between frames
	static String movieFileName = null;
	static String moviePath = null;
	static DecimalFormat fileIdFormat = new DecimalFormat ("#000000000.#;#000000000.#");
	static DecimalFormat timeOutFormat = new DecimalFormat ("#0.00#; #0.00#");	// a clean format for simulation time printing
	

	// for painting control
	static boolean paintOn = true;
	
	// for run control
	static boolean running = false;
	
	public static void main (String args[]) {
		new Sim2D (args);
	}
	
	public Sim2D (String [] inputParams) {
		if (inputParams.length != 0) { parseParams(inputParams); }	// figure out what options have been specified
		// make things to start
		for (int i=0;i<5;i++) { Glutton.makeRandomGlutton(); }
		
		if (!remote) {
			// make the frame to which we draw
			theFrame = new WorldFrame (this, width, height);
			theFrame.showAll();		// draw once at beginning
		}
		
		// start the time loop
		doLoop();
	}
	

		
	public void doLoop() {
		while (simulationTime <= runTime) {
			if (!running) {
				try { Thread.sleep(200); } catch (InterruptedException e) { System.out.println ("error sleeping while paused"); }
			} else {
				for (int i=0;i<Thing.thingCt;i++) {
					Thing.theThings[i].step();
				}
				
				if (!remote) {
					if (paintOn) { 
						theFrame.showAll(); 
						try { Thread.sleep(1); } catch (InterruptedException e) { System.out.println ("error sleeping after paint"); }
					} else { theFrame.showInfoOnly(); }		// only update Thing painting if paintOn
					if (makeMovie) { 
						checkJPEGWrite(); 
						movieCounter++;
					}
				} else {
					checkRemoteReport();
					remoteCounter++;
				}
				
				if (infoWrite) {
					checkInfoWrite();
					infoWriteCounter++;
				}
				
				counter++;
				simulationTime += deltaT;
				
				Thing.removeDeadThings();		// get rid of dead things
				Mutton.spawnMuttons();		// make more muttons...maybe
			}
		}
	}
	
	public void checkJPEGWrite () {
		if (movieCounter >= movieStep) {
			theFrame.showAll(); // paint before save
			String fileName = moviePath + File.separator + movieFileName + String.valueOf(fileIdFormat.format(counter)+".jpg");
			Jpegger.jpegFromBufferedImage(WorldPanel.theImage,fileName);
			movieCounter = 0;
		}
	}
	
	public static void checkRemoteReport () {
		if (remoteCounter >= remoteReportStep) {
			System.out.println(getInfoString());
			remoteCounter = 0;
		}
	}
	
	public static void checkInfoWrite () {
		if (infoWriteCounter >= infoWriteStep) {
			FileOps.writeInfo();
			infoWriteCounter = 0;
		}
	}
	
	public static String getInfoString() {
		String timeStr = " Sim Time = " + String.valueOf(timeOutFormat.format(Sim2D.simulationTime));
		String mutCtStr = " Muttons: " + String.valueOf(Mutton.muttonCt);
		String glutCtStr = " Gluttons: " + String.valueOf(Glutton.gluttonCt);
		return glutCtStr + " , " + mutCtStr + " , " + timeStr;
	}
	
	private void parseParams(String [] inputParams) {
		for (int i = 0; i < inputParams.length; i++) {
			if ((inputParams[i].equals("-help")) | (inputParams[i].equals("?"))) {
				System.out.println ("The following command line arguments are accepted.....");
				System.out.println ("	-help	 prints this help message");
				System.out.println ("	-r remote start (no local GUI) with default initial conditions");
				System.out.println ("-rs [int] remote reporting step interval");
				System.out.println ("	-dt [double] use this value for the fixed time-step");
				System.out.println ("	-if [filename]	write info to the named file");
				System.out.println ("	-is [int] integration steps between info writing");
				System.exit(0);
			}
			if (inputParams[i].equals("-r")) {
				remote = true;
				running = true;
			}
			
			if (inputParams[i].equals("-rs")) {
				Double rStep = Double.valueOf(inputParams[i+1]);
				remoteReportStep = (int) rStep.doubleValue();
			}
			
			if (inputParams[i].equals("-dt")) {
				Double dt = Double.valueOf(inputParams[i+1]);
				deltaT = dt.doubleValue();
				Thing.deltaT = deltaT;
			}
		
			if (inputParams[i].equals("-if")) {
				File infodir = new File(inputParams[i+1]);
				File altdir = new File (inputParams[i+1]);
				int j=1;
				while (altdir.isDirectory()) {
					System.out.println (altdir.getName() + " exists.... keeping file name BUT changing directory name");
					altdir = new File(inputParams[i+1] + "." + String.valueOf(j));
					j++;
				}
				altdir.mkdir();	// make directory for files
				String nameToUse = infodir.getName();
				FileOps.mkInfoFile(altdir.getAbsolutePath() + File.separator + nameToUse);
				//*******
				infoWrite = true;
			}
			
			if (inputParams[i].equals("-is")) {
				Double iStep = Double.valueOf(inputParams[i+1]);
				infoWriteStep = (int) iStep.doubleValue();
			}
			
		}
	}

}
		
		
		
		
		
		
		
		
		
		
		
